unit UCMDateEdt;

{**********************************************************************
 Komponente TCMDateEdt fr Session
 "DL 17 Y2K: Datumsarithmetik mit Delphi"
 Entwicklerkonferenz 1999
 Compiler: DELPHI 4
 ----------------------------------------------------------------------
 component TCMDateEdt for session
 "DL 17 Y2K: Date Arithmetic with Delphi"
 Developer Conference 1999
 Compiler: DELPHI 4
 ----------------------------------------------------------------------

 (c) 1999 by Christian Maas
             www.chmaas.handshake.de
             chmaas@hit.sb.sub.de

 **********************************************************************}

interface
uses StdCtrls, Classes, Controls { nur fr/only for D40 };

type
  TCMDateEdt = class(TEdit)
  private
    FDateYYYYMMDD: string;
    FYear4Digits: boolean;
    FEmptyValid: boolean;
    procedure SetYear4Digits (Value: boolean);
    function  GetValid: boolean;
    function  GetDateYYYYMMDD: string;
  protected
    procedure DoExit; override;
    procedure KeyPress (var Key: Char); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner : TComponent); override;
  published
    { ISO-Datumswert zur Laufzeit / ISO date string at runtime }
    property DateYYYYMMDD: string read GetDateYYYYMMDD;
    { leere Eingabe erlaubt / empty field allowed }
    property EmptyValid: boolean read FEmptyValid write FEmptyValid default true;
    { Gltigkeitsprfung zur Laufzeit / validation at runtime }
    property Valid: boolean read GetValid;
    { Anzeige mit 4-stelligem Jahr / display with 4-digit year }
    property Year4Digits: boolean
      read FYear4Digits write SetYear4Digits default true;
  end;

{**********************************************************************
 Menge von TCMDateEdt-Feldern gltig?
 bergabeparameter A ist ein offenes Array von TCMDateEdt-Komponenten
 bzw. eine Elternkomponente, die TCMDateEdt-Komponenten enthlt.
 Nebeneffekt: das erste ungltige Feld wird fokussiert
 ----------------------------------------------------------------------
 Set of TCMDateEdt-fields valid?
 Parameter A is either an open array of TCMDateEdt components or parent
 component containing TCMDateEdt components.
 Side effect: focus the first invalid field

 Bsp./ex.:
       if not AllDatesValid ([CMDateEdt1, CMDateEdt2, CMDateEdt3]) then
         ShowMessage ('please enter valid date!');

       if not AllDatesValid ([TabSheet1]) then
         ShowMessage ('please enter valid date!');
 **********************************************************************}
function AllDatesValid (A: array of TWinControl): boolean;

procedure Register;

implementation
uses SysUtils, CMDateUtil;

{**********************************************************************
 **********************************************************************}
constructor TCMDateEdt.Create (AOwner : TComponent);
begin
  inherited Create (AOwner);
  MaxLength := 10;
  FDateYYYYMMDD := '';
  FYear4Digits := true;
  FEmptyValid := true;
end;

{**********************************************************************
 **********************************************************************}
procedure TCMDateEdt.SetYear4Digits (Value: boolean);
begin
  if Value <> FYear4Digits then begin
    FYear4Digits := Value;
    { Aufruf der Validierung zur Formatierung der Anzeige /
      call validation to format display }
    GetValid;
  end;
end;

{**********************************************************************
 zentrale Routine fr
 - eigentliche Gltigkeitsprfung
 - Versorgung von FDateYYYYMMDD
 - Formatierung der Anzeige
 ----------------------------------------------------------------------
 core routine for
 - validation itself
 - setting of FDateYYYYMMDD
 - formatting display
 **********************************************************************}
function TCMDateEdt.GetValid: boolean;
var YYYYMMDD: string;
begin
  FDateYYYYMMDD := '';
  if Text = '' then
    Result := EmptyValid
  else begin
    Result := ScanDateString (Text, YYYYMMDD);
    if Result then begin
      if not FYear4Digits and not WithinPivotFrame (ExtractYear (YYYYMMDD)) then
        { Jahr auerhalb des durch Pivot-Jahr definierten Fensters
          dennoch 4-stellig anzeigen /
          display year outside of frame defined by pivot year always
          with 4 digits }
        Text := FormatDate (YYYYMMDD, true)
      else
        Text := FormatDate (YYYYMMDD, FYear4Digits);
      FDateYYYYMMDD := YYYYMMDD;
    end;
  end;
end;

{**********************************************************************
 **********************************************************************}
function TCMDateEdt.GetDateYYYYMMDD: string;
begin
  { Ausnutzung des Nebeneffekts: versorgen von FYYYYMMDD /
    use side effect: setting of FYYYYMMDD }
  GetValid;
  Result := FDateYYYYMMDD;
end;

{**********************************************************************
 Reaktion auf Verlassen des Feldes: Gltigkeitsprfung, Versorgen
 von FDateYYYYMMDD und Formatieren des Feldinhalts
 ----------------------------------------------------------------------
 perform validation, set FDateYYYYMMDD, and format display
 **********************************************************************}
procedure TCMDateEdt.DoExit;
begin
  if not Valid then
    { gesamten Text markieren / mark complete text }
    SelectAll;
  inherited DoExit;
end;

{**********************************************************************
 **********************************************************************}
procedure TCMDateEdt.KeyPress (var Key: Char);
begin
  if (Key = #27) and (FDateYYYYMMDD <> '') then
    { ESC stellt alte Eingabe wieder her / ESC restores old value }
    Text := FormatDate (FDateYYYYMMDD, FYear4Digits)
  else if not (Key in ['0'..'9', DateSeparator, #8, #9]) then
    { erlaubte Tasten: Ziffern, Punkt, <TAB>, <BackSpace> /
      valid keys are: figures, point, <TAB>, <BackSpace> }
    Key := #0;
  inherited KeyPress (Key);
end;

{**********************************************************************
 Umsetzen eines Ausdrucks Now+n bzw. Now-n in ein Datum
 ----------------------------------------------------------------------
 convert string Now+n or Now-n to corresponding date
 **********************************************************************}
procedure TCMDateEdt.Loaded;
var Date: string;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then begin
    if pos ('NOW', UpperCase (Text)) = 1 then begin
      Date := GetCurrentDate;
      if length (Text) > 4 then
        { Teil hinter 'Now' addieren / add part after 'Now' }
        Date := AddDays (Date, StrToInt (copy (Text, 4, length (Text) - 3)));
      Text := FormatDate (Date, Year4Digits);
    end
    else if (length (Text) > 0) and not GetValid then
      { ungltigen Feldinhalt lschen / delete invalid field contents }
      Text := ''
  end;
end;

{**********************************************************************
 Menge von TCMDateEdt-Feldern gltig?
 bergabeparameter A ist ein offenes Array von TCMDateEdt-Komponenten
 bzw. eine Elternkomponente, die TCMDateEdt-Komponenten enthlt.
 Nebeneffekt: das erste ungltige Feld wird fokussiert
 ----------------------------------------------------------------------
 Set of TCMDateEdt-fields valid?
 Parameter A is either an open array of TCMDateEdt components or parent
 component containing TCMDateEdt components.
 Side effect: focus the first invalid field

 Bsp./ex.:
       if not AllDatesValid ([CMDateEdt1, CMDateEdt2, CMDateEdt3]) then
         ShowMessage ('please enter valid date!');

       if not AllDatesValid ([TabSheet1]) then
         ShowMessage ('please enter valid date!');
 **********************************************************************}
function AllDatesValid (A: array of TWinControl): boolean;
var i, j: integer;

  { Validierung einer einzelnen Komponente / validate a single component }
  function IsValid (CMDateEdt: TCMDateEdt): boolean;
  begin
    Result := CMDateEdt.Valid;
    if not Result then
      CMDateEdt.SetFocus;
  end;

begin

  Result := true;
  i := 0;
  while Result and (i <= High (A)) do begin
    if (A [i] is TCMDateEdt) then
      Result := IsValid ((A [i] as TCMDateEdt))
    else begin
      { Schleife ber Unterkomponenten / search subcomponents of parent }
      j := 0;
      while Result and (j <= A [i].ComponentCount - 1) do begin
        if A [i].Components [j] is TCMDateEdt then
          Result := IsValid (A [i].Components [j] as TCMDateEdt);
        inc (j);
      end;
    end;
    inc (i);
  end;

end;

{**********************************************************************
 **********************************************************************}
procedure Register;
begin
  RegisterComponents ('CM', [TCMDateEdt]);
end;

end.
